@@ -1,7 +1,7 @@
 #' Takes an emtools.species object with presence and background points, and builds a Domain model
 #'
 #' @param species An enmtools.species object
-#' @param env A raster or raster stack of environmental data.
+#' @param env A SpatRaster of environmental data.
 #' @param test.prop Proportion of data to withhold randomly for model evaluation, or "block" for spatially structured evaluation.
 #' @param report Optional name of an html file for generating reports
 #' @param overwrite TRUE/FALSE whether to overwrite a report file if it already exists
@@ -19,8 +19,6 @@
 #'
 #' @examples
 #' \donttest{
-#' data(euro.worldclim)
-#' data(iberolacerta.clade)
 #' enmtools.dm(iberolacerta.clade$species$monticola, env = euro.worldclim)
 #' }
 
@@ -29,6 +27,8 @@ enmtools.dm <- function(species, env = NA, test.prop = 0, report = NULL, nback =
 
   notes <- NULL
 
+  env <- check.raster(env, "env")
+
   species <- check.bg(species, env, nback, verbose = verbose, bias = bias)
 
   dm.precheck(species, env)
@@ -55,7 +55,7 @@ enmtools.dm <- function(species, env = NA, test.prop = 0, report = NULL, nback =
       } else if(corner < 1 | corner > 4){
         stop("corner should be an integer from 1 to 4!")
       }
-      test.inds <- get.block(species$presence.points, species$background.points)
+      test.inds <- get.block(terra::crds(species$presence.points), terra::crds(species$background.points))
       test.bg.inds <- which(test.inds$bg.grp == corner)
       test.inds <- which(test.inds$occs.grp == corner)
       test.data <- species$presence.points[test.inds,]
@@ -69,40 +69,41 @@ enmtools.dm <- function(species, env = NA, test.prop = 0, report = NULL, nback =
   # fail if the stack only has one layer.
   if(length(names(env)) == 1){
     oldname <- names(env)
-    env <- stack(env, env)
+    env <- c(env, env)
     env[[2]][!is.na(env[[2]])] <- 0
     names(env) <- c(oldname, "dummyvar")
     notes <- c(notes, "Only one predictor was provided, so a dummy variable was created in order to be compatible with dismo's prediction function.")
   }
 
+  # dismo's domain function doesn't work with SpatRaster
+  # objects so we're going to convert to a matrix instead
+  this.dm <- dismo::domain(terra::extract(env, species$presence.points, ID = FALSE))
 
-  this.dm <- dismo::domain(env, species$presence.points[,1:2])
-
-  suitability <- predict(env, this.dm, type = "response")
+  suitability <- terra::predict(env, this.dm, type = "response", na.rm = TRUE)
 
   # Clamping and getting a diff layer
   clamping.strength <- NA
   if(clamp == TRUE){
     # Adding env (skipped for BC otherwise)
-    this.df <- as.data.frame(extract(env, species$presence.points))
+    this.df <- as.data.frame(terra::extract(env, species$presence.points, ID = FALSE))
 
     env <- clamp.env(this.df, env)
-    clamped.suitability <- predict(env, this.dm, type = "response")
+    clamped.suitability <- terra::predict(env, this.dm, type = "response", na.rm = TRUE)
     clamping.strength <- clamped.suitability - suitability
     suitability <- clamped.suitability
   }
 
-  model.evaluation <- dismo::evaluate(species$presence.points[,1:2], species$background.points[,1:2],
-                               this.dm, env)
+  model.evaluation <- dismo::evaluate(species$presence.points, species$background.points,
+                               this.dm, env, na.rm = TRUE)
   env.model.evaluation <- env.evaluate(species, this.dm, env, n.background = env.nback)
 
   # Test eval for randomly withheld data
   if(is.numeric(test.prop)){
     if(test.prop > 0 & test.prop < 1){
-      test.check <- raster::extract(env, test.data)
+      test.check <- terra::extract(env, test.data, ID = FALSE)
       test.data <- test.data[complete.cases(test.check),]
-      test.evaluation <-dismo::evaluate(test.data, species$background.points[,1:2],
-                                        this.dm, env)
+      test.evaluation <-dismo::evaluate(test.data, species$background.points,
+                                        this.dm, env, na.rm = TRUE)
       temp.sp <- species
       temp.sp$presence.points <- test.data
       env.test.evaluation <- env.evaluate(temp.sp, this.dm, env, n.background = env.nback)
@@ -112,10 +113,10 @@ enmtools.dm <- function(species, env = NA, test.prop = 0, report = NULL, nback =
   # Test eval for spatially structured data
   if(is.character(test.prop)){
     if(test.prop == "block"){
-      test.check <- raster::extract(env, test.data)
+      test.check <- terra::extract(env, test.data, ID = FALSE)
       test.data <- test.data[complete.cases(test.check),]
       test.evaluation <-dismo::evaluate(test.data, test.bg,
-                                        this.dm, env)
+                                        this.dm, env, na.rm = TRUE)
       temp.sp <- species
       temp.sp$presence.points <- test.data
       temp.sp$background.points <- test.bg
@@ -158,11 +159,19 @@ enmtools.dm <- function(species, env = NA, test.prop = 0, report = NULL, nback =
       rep.species <- species
 
       # Mix the points all together
-      allpoints <- rbind(test.data, species$background.points, species$presence.points)
+      # Mix the points all together
+        if(test.prop > 0) {
+          test <- as.data.frame(test.data, geom = "XY")[ , c("x", "y")]
+        } else {
+          test <- NULL
+        }
+        allpoints <- rbind(test,
+                           as.data.frame(species$background.points, geom = "XY")[ , c("x", "y")],
+                           as.data.frame(species$presence.points, geom = "XY")[ , c("x", "y")])
 
       # Sample presence points from pool and remove from pool
       rep.rows <- sample(nrow(allpoints), nrow(species$presence.points))
-      rep.species$presence.points <- allpoints[rep.rows,]
+      rep.species$presence.points <- terra::vect(allpoints[rep.rows,], geom = c("x", "y"), crs = terra::crs(species$presence.points))
       allpoints <- allpoints[-rep.rows,]
 
       # Do the same for test points
@@ -173,32 +182,39 @@ enmtools.dm <- function(species, env = NA, test.prop = 0, report = NULL, nback =
       }
 
       # Everything else goes back to the background
-      rep.species$background.points <- allpoints
+      rep.species$background.points <- terra::vect(allpoints, geom = c("x", "y"), crs = terra::crs(species$presence.points))
 
-      thisrep.dm <- dismo::domain(env, rep.species$presence.points[,1:2])
+      thisrep.dm <- dismo::domain(terra::extract(env, rep.species$presence.points, ID = FALSE))
 
-      thisrep.model.evaluation <-dismo::evaluate(rep.species$presence.points[,1:2], rep.species$background.points[,1:2],
-                                                 thisrep.dm, env)
+      thisrep.model.evaluation <-dismo::evaluate(rep.species$presence.points, rep.species$background.points,
+                                                 thisrep.dm, env, na.rm = TRUE)
       thisrep.env.model.evaluation <- env.evaluate(rep.species, thisrep.dm, env, n.background = env.nback)
 
       rts.geog.training[i] <- thisrep.model.evaluation@auc
       rts.env.training[i] <- thisrep.env.model.evaluation@auc
 
       if(test.prop > 0 & test.prop < 1){
-        thisrep.test.evaluation <-dismo::evaluate(rep.test.data, rep.species$background.points[,1:2],
-                                                  thisrep.dm, env)
+        thisrep.test.evaluation <-dismo::evaluate(rep.test.data, rep.species$background.points,
+                                                  thisrep.dm, env, na.rm = TRUE)
         temp.sp <- rep.species
-        temp.sp$presence.points <- rep.test.data
+        temp.sp$presence.points <- terra::vect(rep.test.data, geom = c("x", "y"), crs = terra::crs(species$presence.points))
         thisrep.env.test.evaluation <- env.evaluate(temp.sp, thisrep.dm, env, n.background = env.nback)
 
         rts.geog.test[i] <- thisrep.test.evaluation@auc
         rts.env.test[i] <- thisrep.env.test.evaluation@auc
+
+        rts.models[[paste0("rep.",i)]] <- list(model = thisrep.dm,
+                                               training.evaluation = thisrep.model.evaluation,
+                                               env.training.evaluation = thisrep.env.model.evaluation,
+                                               test.evaluation = thisrep.test.evaluation,
+                                               env.test.evaluation = thisrep.env.test.evaluation)
+      } else {
+        rts.models[[paste0("rep.",i)]] <- list(model = thisrep.dm,
+                                               training.evaluation = thisrep.model.evaluation,
+                                               env.training.evaluation = thisrep.env.model.evaluation,
+                                               test.evaluation = NA,
+                                               env.test.evaluation = NA)
       }
-      rts.models[[paste0("rep.",i)]] <- list(model = thisrep.dm,
-                                             training.evaluation = thisrep.model.evaluation,
-                                             env.training.evaluation = thisrep.env.model.evaluation,
-                                             test.evaluation = thisrep.test.evaluation,
-                                             env.test.evaluation = thisrep.env.test.evaluation)
     }
 
     # Reps are all run now, time to package it all up
@@ -214,31 +230,40 @@ enmtools.dm <- function(species, env = NA, test.prop = 0, report = NULL, nback =
       rts.env.test.pvalue <- NA
     }
 
+    rts.geog.training <- data.frame(AUC = rts.geog.training)
+    rts.env.training <- data.frame(AUC = rts.env.training)
+    rts.geog.test <- data.frame(AUC = rts.geog.test)
+    rts.env.test <- data.frame(AUC = rts.env.test)
+
     # Making plots
-    training.plot <- qplot(rts.geog.training, geom = "histogram", fill = "density", alpha = 0.5) +
+    training.plot <- ggplot(rts.geog.training, aes(x = .data$AUC, fill = "density", alpha = 0.5)) +
+      geom_histogram(binwidth = 0.05) +
       geom_vline(xintercept = model.evaluation@auc, linetype = "longdash") +
-      xlim(0,1) + guides(fill = FALSE, alpha = FALSE) + xlab("AUC") +
+      xlim(-0.05,1.05) + guides(fill = "none", alpha = "none") + xlab("AUC") +
       ggtitle(paste("Model performance in geographic space on training data")) +
       theme(plot.title = element_text(hjust = 0.5))
 
-    env.training.plot <- qplot(rts.env.training, geom = "histogram", fill = "density", alpha = 0.5) +
-      geom_vline(xintercept = env.model.evaluation@auc, linetype = "longdash") +
-      xlim(0,1) + guides(fill = FALSE, alpha = FALSE) + xlab("AUC") +
-      ggtitle(paste("Model performance in environmental space on training data")) +
+    env.training.plot <- ggplot(rts.env.training, aes(x = .data$AUC, fill = "density", alpha = 0.5)) +
+      geom_histogram(binwidth = 0.05) +
+      geom_vline(xintercept = model.evaluation@auc, linetype = "longdash") +
+      xlim(-0.05,1.05) + guides(fill = "none", alpha = "none") + xlab("AUC") +
+      ggtitle(paste("Model performance in environment space on training data")) +
       theme(plot.title = element_text(hjust = 0.5))
 
     # Make plots for test AUC distributions
     if(test.prop > 0){
-      test.plot <- qplot(rts.geog.test, geom = "histogram", fill = "density", alpha = 0.5) +
-        geom_vline(xintercept = test.evaluation@auc, linetype = "longdash") +
-        xlim(0,1) + guides(fill = FALSE, alpha = FALSE) + xlab("AUC") +
+      test.plot <- ggplot(rts.geog.test, aes(x = .data$AUC, fill = "density", alpha = 0.5)) +
+        geom_histogram(binwidth = 0.05) +
+        geom_vline(xintercept = model.evaluation@auc, linetype = "longdash") +
+        xlim(-0.05,1.05) + guides(fill = "none", alpha = "none") + xlab("AUC") +
         ggtitle(paste("Model performance in geographic space on test data")) +
         theme(plot.title = element_text(hjust = 0.5))
 
-      env.test.plot <- qplot(rts.env.test, geom = "histogram", fill = "density", alpha = 0.5) +
-        geom_vline(xintercept = env.test.evaluation@auc, linetype = "longdash") +
-        xlim(0,1) + guides(fill = FALSE, alpha = FALSE) + xlab("AUC") +
-        ggtitle(paste("Model performance in environmental space on test data")) +
+      env.test.plot <- ggplot(rts.env.test, aes(x = .data$AUC, fill = "density", alpha = 0.5)) +
+        geom_histogram(binwidth = 0.05) +
+        geom_vline(xintercept = model.evaluation@auc, linetype = "longdash") +
+        xlim(-0.05,1.05) + guides(fill = "none", alpha = "none") + xlab("AUC") +
+        ggtitle(paste("Model performance in environment space on test data")) +
         theme(plot.title = element_text(hjust = 0.5))
     } else {
       test.plot <- NA
@@ -265,8 +290,11 @@ enmtools.dm <- function(species, env = NA, test.prop = 0, report = NULL, nback =
                      rts.nreps = rts.reps)
   }
 
+  analysis.df <- cbind(terra::crds(species$presence.points),
+                       terra::extract(env, species$presence.points, as.points = TRUE, ID = FALSE))
+
   output <- list(species.name = species$species.name,
-                 analysis.df = species$presence.points[,1:2],
+                 analysis.df = analysis.df,
                  test.data = test.data,
                  test.prop = test.prop,
                  model = this.dm,
@@ -308,7 +336,7 @@ message("This function not enabled yet.  Check back soon!")
 
 
 # Summary for objects of class enmtools.dm
-summary.enmtools.dm <- function(object, ...){
+summary.enmtools.dm <- function(object, plot = TRUE, ...){
 
   cat("\n\nData table (top ten lines): ")
   print(kable(head(object$analysis.df, 10)))
@@ -337,7 +365,9 @@ summary.enmtools.dm <- function(object, ...){
   cat("\n\nNotes:  \n")
   print(object$notes)
 
-  plot(object)
+  if(plot) {
+    plot(object)
+  }
 
 }
 
@@ -345,29 +375,30 @@ summary.enmtools.dm <- function(object, ...){
 #Print method for objects of class enmtools.dm
 print.enmtools.dm <- function(x, ...){
 
-  print(summary(x))
+  print(summary(x, ...))
 
 }
 
 # Plot method for objects of class enmtools.dm
 plot.enmtools.dm <- function(x, ...){
 
-  suit.points <- data.frame(rasterToPoints(x$suitability))
-  colnames(suit.points) <- c("Longitude", "Latitude", "Suitability")
+  suit.points <- data.frame(rasterToPoints2(x$suitability))
+  colnames(suit.points) <- c("x", "y", "Suitability")
+  test <- terra::as.data.frame(x$test.data, geom = "XY")
 
-  suit.plot <- ggplot(data = suit.points,  aes_string(y = "Latitude", x = "Longitude")) +
-    geom_raster(aes_string(fill = "Suitability")) +
+  suit.plot <- ggplot(data = suit.points,  aes(y = .data$y, x = .data$x)) +
+    geom_raster(aes(fill = .data$Suitability)) +
     scale_fill_viridis_c(option = "B", guide = guide_colourbar(title = "Suitability")) +
     coord_fixed() + theme_classic() +
     geom_point(data = x$analysis.df[x$analysis.df$presence == 1 &
-                                      x$analysis.df$Longitude > extent(x$suitability)[1] &
-                                      x$analysis.df$Longitude < extent(x$suitability)[2] &
-                                      x$analysis.df$Latitude > extent(x$suitability)[3] &
-                                      x$analysis.df$Latitude < extent(x$suitability)[4],], aes_string(y = "Latitude", x = "Longitude"),
+                                      x$analysis.df$x > terra::ext(x$suitability)[1] &
+                                      x$analysis.df$x < terra::ext(x$suitability)[2] &
+                                      x$analysis.df$y > terra::ext(x$suitability)[3] &
+                                      x$analysis.df$y < terra::ext(x$suitability)[4],], aes(y = .data$y, x = .data$x),
                pch = 21, fill = "white", color = "black", size = 2)
 
-  if(!(all(is.na(x$test.data)))){
-    suit.plot <- suit.plot + geom_point(data = x$test.data,  aes_string(y = "Latitude", x = "Longitude"),
+  if(inherits(x$test.data, "SpatVector")){
+    suit.plot <- suit.plot + geom_point(data = test,  aes(y = .data$y, x = .data$x),
                                         pch = 21, fill = "green", color = "black", size = 2)
   }
 
@@ -389,32 +420,32 @@ predict.enmtools.dm <- function(object, env, maxpts = 1000, clamp = TRUE, ...){
   env <- env[[colnames(object$model@presence)]]
 
   # Make a plot of habitat suitability in the new region
-  suitability <- raster::predict(env, object$model)
+  suitability <- terra::predict(env, object$model, na.rm = TRUE)
 
   # Clamping and getting a diff layer
   clamping.strength <- NA
   if(clamp == TRUE){
     # Adding env (skipped for BC otherwise)
-    this.df <- as.data.frame(extract(env, object$analysis.df))
+    this.df <- as.data.frame(terra::extract(env, object$analysis.df, ID = FALSE))
 
     env <- clamp.env(this.df, env)
-    clamped.suitability <- raster::predict(env, object$model)
+    clamped.suitability <- terra::predict(env, object$model, na.rm = TRUE)
     clamping.strength <- clamped.suitability - suitability
     suitability <- clamped.suitability
   }
 
-  suit.points <- data.frame(rasterToPoints(suitability))
-  colnames(suit.points) <- c("Longitude", "Latitude", "Suitability")
+  suit.points <- data.frame(rasterToPoints2(suitability))
+  colnames(suit.points) <- c("x", "y", "Suitability")
 
-  suit.plot <- ggplot(data = suit.points,  aes_string(y = "Latitude", x = "Longitude")) +
-    geom_raster(aes_string(fill = "Suitability")) +
+  suit.plot <- ggplot(data = suit.points,  aes(y = .data$y, x = .data$x)) +
+    geom_raster(aes(fill = .data$Suitability)) +
     scale_fill_viridis_c(option = "B", guide = guide_colourbar(title = "Suitability")) +
     coord_fixed() + theme_classic()
 
-  clamp.points <- data.frame(rasterToPoints(clamping.strength))
-  colnames(clamp.points) <- c("Longitude", "Latitude", "Clamping")
+  clamp.points <- data.frame(rasterToPoints2(clamping.strength))
+  colnames(clamp.points) <- c("x", "y", "Clamping")
 
-  clamp.plot <- ggplot(data = clamp.points,  aes_string(y = "Latitude", x = "Longitude")) +
+  clamp.plot <- ggplot(data = clamp.points,  aes(y = .data$y, x = .data$x)) +
     geom_raster(aes_string(fill = "Clamping")) +
     scale_fill_viridis_c(option = "B", guide = guide_colourbar(title = "Suitability")) +
     coord_fixed() + theme_classic()
@@ -445,17 +476,13 @@ dm.precheck <- function(species, env, f){
 
   check.species(species)
 
-  if(!inherits(species$presence.points, "data.frame")){
-    stop("Species presence.points do not appear to be an object of class data.frame")
+  if(!inherits(species$presence.points, "SpatVector")){
+    stop("Species presence.points do not appear to be an object of class SpatVector")
   }
 
-  if(!inherits(env, c("raster", "RasterLayer", "RasterStack", "RasterBrick"))){
+  if(!inherits(env, c("SpatRaster"))){
     stop("No environmental rasters were supplied!")
   }
-
-  if(ncol(species$presence.points) != 2){
-    stop("Species presence points do not contain longitude and latitude data!")
-  }
 }
 
 
