@@ -15,8 +15,6 @@
 #'
 #' @examples
 #' \donttest{
-#' data(iberolacerta.clade)
-#' data(euro.worldclim)
 #' cyreni <- iberolacerta.clade$species$cyreni
 #' monticola <- iberolacerta.clade$species$monticola
 #' cyreni.glm <- enmtools.glm(cyreni, euro.worldclim, f = pres ~ bio1 + bio12, nback = 500)
@@ -68,9 +66,10 @@ env.overlap <- function(model.1, model.2, env, tolerance = .001, max.reps = 10,
     this.lhs <- randomLHS(chunk.size, length(names(env)))
 
     # Setting it up so we can handle either a set of rasters or a list of minima and maxima
-    if(inherits(env, c("raster", "RasterStack", "RasterBrick", "RasterLayer"))){
-      mins <- minValue(env)
-      maxes <- maxValue(env)
+    if(inherits(env, c("SpatRaster"))){
+      minmax <- terra::minmax(env)
+      mins <- minmax[1,]
+      maxes <- minmax[2,]
     } else if (inherits(env, "list")){
       mins <- unlist(lapply(env, min))
       maxes <- unlist(lapply(env, max))
@@ -122,7 +121,7 @@ env.overlap <- function(model.1, model.2, env, tolerance = .001, max.reps = 10,
     # First we'll do it if both are recalibrated
     if(!all(is.na(recal.model.1)) & !all(is.na(recal.model.2))){
 
-      check.packages("CalibratR")
+      assert.extras.this.fun()
 
       recal.names <- intersect(names(recal.model.1$recalibrated.model$predictions),
                                names(recal.model.2$recalibrated.model$predictions))
@@ -143,7 +142,7 @@ env.overlap <- function(model.1, model.2, env, tolerance = .001, max.reps = 10,
     # Now if just model 1 is recalibrated
     if(!all(is.na(recal.model.1)) & all(is.na(recal.model.2))){
 
-      check.packages("CalibratR")
+      assert.extras.this.fun()
 
       recal.names <- names(recal.model.1$recalibrated.model$predictions)
 
@@ -163,7 +162,7 @@ env.overlap <- function(model.1, model.2, env, tolerance = .001, max.reps = 10,
     # Now if just model 2 is recalibrated
     if(all(is.na(recal.model.1)) & !all(is.na(recal.model.2))){
 
-      check.packages("CalibratR")
+      assert.extras.this.fun()
 
       recal.names <- names(recal.model.2$recalibrated.model$predictions)
 
@@ -259,7 +258,7 @@ env.overlap <- function(model.1, model.2, env, tolerance = .001, max.reps = 10,
       # First we'll do it if both are recalibrated
       if(!all(is.na(recal.model.1)) & !all(is.na(recal.model.2))){
 
-        check.packages("CalibratR")
+        assert.extras.this.fun()
 
         recal.names <- intersect(names(recal.model.1$recalibrated.model$predictions),
                                  names(recal.model.2$recalibrated.model$predictions))
@@ -280,7 +279,7 @@ env.overlap <- function(model.1, model.2, env, tolerance = .001, max.reps = 10,
       # Now if just model 1 is recalibrated
       if(!all(is.na(recal.model.1)) & all(is.na(recal.model.2))){
 
-        check.packages("CalibratR")
+        assert.extras.this.fun()
 
         recal.names <- names(recal.model.1$recalibrated.model$predictions)
 
@@ -300,7 +299,7 @@ env.overlap <- function(model.1, model.2, env, tolerance = .001, max.reps = 10,
       # Now if just model 2 is recalibrated
       if(all(is.na(recal.model.1)) & !all(is.na(recal.model.2))){
 
-        check.packages("CalibratR")
+        assert.extras.this.fun()
 
         recal.names <- names(recal.model.2$recalibrated.model$predictions)
 
@@ -351,22 +350,34 @@ env.overlap <- function(model.1, model.2, env, tolerance = .001, max.reps = 10,
 
   output <- NA
 
+  plot.df <- data.frame(gens = gens,
+                        D = this.d,
+                        I = this.i,
+                        cor = this.cor)
+
+
   # Packing list for non-recalibrated models
   if(all(is.na(recal.model.1)) & all(is.na(recal.model.2))){
     output <- list(env.D = mean(this.d),
                    env.I = mean(this.i),
                    env.cor = mean(this.cor),
-                   env.D.plot = qplot(gens, this.d, ylab = "D", xlab = "Samples", ylim = c(0,1)),
-                   env.I.plot = qplot(gens, this.i, ylab = "I", xlab = "Samples", ylim = c(0,1)),
-                   env.cor.plot = qplot(gens, this.cor, ylab = "Correlation", xlab = "Samples", ylim = c(-1,1)))
+                   env.D.plot = ggplot(data = plot.df, aes(x = .data$gens, y = .data$D)) + geom_point() +
+                     ylab("D") + xlab("Samples") + ylim(0,1),
+                   env.I.plot = ggplot(data = plot.df, aes(x = .data$gens, y = .data$I)) + geom_point() +
+                     ylab("I") + xlab("Samples") + ylim(0,1),
+                   env.cor.plot = ggplot(data = plot.df, aes(x = .data$gens, y = .data$cor)) + geom_point() +
+                     ylab("Rank Correlation") + xlab("Samples") + ylim(-1,1))
   } else {
     # At least one of the models was recalibrated
     output <- list(env.D = mean(this.d),
                    env.I = mean(this.i),
                    env.cor = mean(this.cor),
-                   env.D.plot = qplot(gens, this.d, ylab = "D", xlab = "Samples", ylim = c(0,1)),
-                   env.I.plot = qplot(gens, this.i, ylab = "I", xlab = "Samples", ylim = c(0,1)),
-                   env.cor.plot = qplot(gens, this.cor, ylab = "Correlation", xlab = "Samples", ylim = c(-1,1)),
+                   env.D.plot = ggplot(data = plot.df, aes(x = .data$gens, y = .data$D)) + geom_point() +
+                     ylab("D") + xlab("Samples") + ylim(0,1),
+                   env.I.plot = ggplot(data = plot.df, aes(x = .data$gens, y = .data$I)) + geom_point() +
+                     ylab("I") + xlab("Samples") + ylim(0,1),
+                   env.cor.plot = ggplot(data = plot.df, aes(x = .data$gens, y = .data$cor)) + geom_point() +
+                     ylab("Rank Correlation") + xlab("Samples") + ylim(-1,1),
                    recal.env.D = lapply(recal.this.d, function(x) mean(x)),
                    recal.env.i = lapply(recal.this.i, function(x) mean(x)),
                    recal.env.cor = lapply(recal.this.cor, function(x) mean(x)))
